# import the required libraries
import numpy as np
import matplotlib.pyplot as plt

def dsig_dy(xx,yy):
        result=1-yy+1/(1-yy)-4*yy/xx/(1-yy)+4*pow(yy/xx/(1-yy),2)
        return result

def computeSpectrum_brute_force(xx, nPoints=10000, nBins=50, seed=None):
        ym = xx/(xx+1)
        dsig_dy_val_max=dsig_dy(xx,ym)
        nFunctionEval = 0
        yy_rej_method = []
        generator = np.random.RandomState(seed=seed)
        while len(yy_rej_method) < nPoints:
                rr = generator.uniform(size=2)
                yy = rr[0]*ym
                nFunctionEval += 1
                dsig_dy_val=dsig_dy(xx,yy)
                if(dsig_dy_val > dsig_dy_val_max):
                        print(' dsig_dy_val= ',dsig_dy_val,' exceeds dsig_dy_val_max= ',dsig_dy_val_max,' program will now exit ')
                        exit(99)
                if(dsig_dy_val/dsig_dy_val_max > rr[1]) :
                        yy_rej_method.append(yy)
        plt.hist(yy_rej_method, bins=nBins,range=(0.,1.2))
        plt.title(r'$\mathrm{compton Conversion Rej Method}\ x=%.3f,\ ym=%.3f$' %(xx, ym))
        print(' rejection method xx= ',xx,' nPoints= ',nPoints,' nFunctionEval= ',nFunctionEval)
        plt.show()

def computeSpectrum_hybrid(xx, nPoints=10000, nBins=50, seed=None):
        ym = xx/(xx+1)
        dsig_dy_val_max=dsig_dy(xx,ym)
        ratio_max=max(dsig_dy(xx,0),dsig_dy_val_max*(1-ym))
        print(' xx= ',xx,' ratio_max= ',ratio_max)
        nFunctionEval = 0
        yy_hybrid_method = []
        generator = np.random.RandomState(seed=seed)
        while len(yy_hybrid_method) < nPoints:
                rr = generator.uniform(size=2)
                yy = 1 - (1 - ym)**rr[0]
                nFunctionEval += 1
                dsig_dy_val = dsig_dy(xx,yy)
                ratio = dsig_dy_val*(1 - yy)
                if(ratio > ratio_max):
                        print(' ratio= ',ratio,' exceeds ratio_max= ',ratio_max,' ym= ',ym,' yy= ',yy,' program will now exit ')
                        exit(99)
                if(ratio/ratio_max > rr[1]) :
                        yy_hybrid_method.append(yy)
        plt.hist(yy_hybrid_method, bins=nBins,range=(0.,1.2))
        plt.title(r'$\mathrm{compton Conversion Hybrid Method}\ x=%.3f,\ ym=%.3f$' %(xx, ym))
        print(' hybrid method xx= ',xx,' nPoints= ',nPoints,' nFunctionEval= ',nFunctionEval)
        plt.show()

xxVec = [3.13, 1000., 100000.]
for xx in xxVec:
        computeSpectrum_brute_force(xx)
        computeSpectrum_hybrid(xx)